<?php
/**
 * Copyright ©  All rights reserved.
 * See COPYING.txt for license details.
 */
declare(strict_types=1);

namespace Ymina\HomeDelivery\Model;

use Magento\Framework\Api\SearchCriteria\CollectionProcessorInterface;
use Magento\Framework\Exception\CouldNotDeleteException;
use Magento\Framework\Exception\CouldNotSaveException;
use Magento\Framework\Exception\NoSuchEntityException;
use Ymina\HomeDelivery\Api\Data\FeeInterface;
use Ymina\HomeDelivery\Api\Data\FeeInterfaceFactory;
use Ymina\HomeDelivery\Api\Data\FeeSearchResultsInterfaceFactory;
use Ymina\HomeDelivery\Api\FeeRepositoryInterface;
use Ymina\HomeDelivery\Model\ResourceModel\Fee as ResourceFee;
use Ymina\HomeDelivery\Model\ResourceModel\Fee\CollectionFactory as FeeCollectionFactory;

class FeeRepository implements FeeRepositoryInterface
{

    /**
     * @var Fee
     */
    protected $searchResultsFactory;

    /**
     * @var FeeInterfaceFactory
     */
    protected $feeFactory;

    /**
     * @var FeeCollectionFactory
     */
    protected $feeCollectionFactory;

    /**
     * @var CollectionProcessorInterface
     */
    protected $collectionProcessor;

    /**
     * @var ResourceFee
     */
    protected $resource;


    /**
     * @param ResourceFee $resource
     * @param FeeInterfaceFactory $feeFactory
     * @param FeeCollectionFactory $feeCollectionFactory
     * @param FeeSearchResultsInterfaceFactory $searchResultsFactory
     * @param CollectionProcessorInterface $collectionProcessor
     */
    public function __construct(
        ResourceFee $resource,
        FeeInterfaceFactory $feeFactory,
        FeeCollectionFactory $feeCollectionFactory,
        FeeSearchResultsInterfaceFactory $searchResultsFactory,
        CollectionProcessorInterface $collectionProcessor
    ) {
        $this->resource = $resource;
        $this->feeFactory = $feeFactory;
        $this->feeCollectionFactory = $feeCollectionFactory;
        $this->searchResultsFactory = $searchResultsFactory;
        $this->collectionProcessor = $collectionProcessor;
    }

    /**
     * @inheritDoc
     */
    public function save(FeeInterface $fee)
    {
        try {
            $this->resource->save($fee);
        } catch (\Exception $exception) {
            throw new CouldNotSaveException(__(
                'Could not save the fee: %1',
                $exception->getMessage()
            ));
        }
        return $fee;
    }

    /**
     * @inheritDoc
     */
    public function get($feeId)
    {
        $fee = $this->feeFactory->create();
        $this->resource->load($fee, $feeId);
        if (!$fee->getId()) {
            throw new NoSuchEntityException(__('Fee with id "%1" does not exist.', $feeId));
        }
        return $fee;
    }

    /**
     * @inheritDoc
     */
    public function getList(
        \Magento\Framework\Api\SearchCriteriaInterface $criteria
    ) {
        $collection = $this->feeCollectionFactory->create();
        
        $this->collectionProcessor->process($criteria, $collection);
        
        $searchResults = $this->searchResultsFactory->create();
        $searchResults->setSearchCriteria($criteria);
        
        $items = [];
        foreach ($collection as $model) {
            $items[] = $model;
        }
        
        $searchResults->setItems($items);
        $searchResults->setTotalCount($collection->getSize());
        return $searchResults;
    }

    /**
     * @inheritDoc
     */
    public function delete(FeeInterface $fee)
    {
        try {
            $feeModel = $this->feeFactory->create();
            $this->resource->load($feeModel, $fee->getFeeId());
            $this->resource->delete($feeModel);
        } catch (\Exception $exception) {
            throw new CouldNotDeleteException(__(
                'Could not delete the Fee: %1',
                $exception->getMessage()
            ));
        }
        return true;
    }

    /**
     * @inheritDoc
     */
    public function deleteById($feeId)
    {
        return $this->delete($this->get($feeId));
    }
}

